@@ -6,6 +6,15 @@
 #' The corresponding HTML output tag should be \code{div} or \code{img} and have
 #' the CSS class name \code{shiny-plot-output}.
 #'
+#' @section Interactive plots:
+#'
+#'   With ggplot2 graphics, the code in \code{renderPlot} should return a ggplot
+#'   object; if instead the code prints the ggplot2 object with something like
+#'   \code{print(p)}, then the coordinates for interactive graphics will not be
+#'   properly scaled to the data space.
+#'
+#'   See \code{\link{plotOutput}} for more information about interactive plots.
+#'
 #' @seealso For the corresponding client-side output function, and example
 #'   usage, see \code{\link{plotOutput}}. For more details on how the plots are
 #'   generated, and how to control the output, see \code{\link{plotPNG}}.
@@ -32,11 +41,9 @@
 #' @export
 renderPlot <- function(expr, width='auto', height='auto', res=72, ...,
                        env=parent.frame(), quoted=FALSE, func=NULL) {
-  if (!is.null(func)) {
-    shinyDeprecated(msg="renderPlot: argument 'func' is deprecated. Please use 'expr' instead.")
-  } else {
-    installExprFunction(expr, "func", env, quoted)
-  }
+  # This ..stacktraceon is matched by a ..stacktraceoff.. when plotFunc
+  # is called
+  installExprFunction(expr, "func", env, quoted, ..stacktraceon = TRUE)
 
   args <- list(...)
 
@@ -95,7 +102,9 @@ renderPlot <- function(expr, width='auto', height='auto', res=72, ...,
         if (inherits(result$value, "ggplot")) {
           utils::capture.output(coordmap <<- getGgplotCoordmap(result$value, pixelratio))
         } else {
-          utils::capture.output(print(result$value))
+          # This ..stacktraceon.. negates the ..stacktraceoff.. that wraps the
+          # call to plotFunc
+          utils::capture.output(..stacktraceon..(print(result$value)))
         }
       }
 
@@ -104,8 +113,14 @@ renderPlot <- function(expr, width='auto', height='auto', res=72, ...,
       }
     }
 
-    outfile <- do.call(plotPNG, c(plotFunc, width=width*pixelratio,
-                                  height=height*pixelratio, res=res*pixelratio, args))
+    # This ..stacktraceoff.. is matched by the `func` function's
+    # wrapFunctionLabel(..stacktraceon=TRUE) call near the beginning of
+    # renderPlot, and by the ..stacktraceon.. in plotFunc where ggplot objects
+    # are printed
+    outfile <- ..stacktraceoff..(
+      do.call(plotPNG, c(plotFunc, width=width*pixelratio,
+        height=height*pixelratio, res=res*pixelratio, args))
+    )
     on.exit(unlink(outfile))
 
     # A list of attributes for the img
@@ -482,9 +497,20 @@ getGgplotCoordmap <- function(p, pixelratio) {
     # are "null" units. These units use the remaining available width/height --
     # that is, the space not occupied by elements that have an absolute size.
     is_null_unit <- function(x) {
-      vapply(x, FUN.VALUE = logical(1), function(u) {
-        isTRUE(attr(u, "unit", exact = TRUE) == "null")
-      })
+      # A vector of units can be either a list of individual units (a unit.list
+      # object), each with their own set of attributes, or an atomic vector with
+      # one set of attributes. ggplot2 switched from the former (in version
+      # 1.0.1) to the latter. We need to make sure that we get the correct
+      # result in both cases.
+      if (inherits(x, "unit.list")) {
+        # For ggplot2 <= 1.0.1
+        vapply(x, FUN.VALUE = logical(1), function(u) {
+          isTRUE(attr(u, "unit", exact = TRUE) == "null")
+        })
+      } else {
+        # For later versions of ggplot2
+        attr(x, "unit", exact = TRUE) == "null"
+      }
     }
 
     # Convert a unit (or vector of units) to a numeric vector of pixel sizes
