@@ -11,7 +11,9 @@ shinyServer(function(input, output) {
   })
   
   output$downloadData <- downloadHandler(
-    filename = function() { paste(input$dataset, '.csv', sep='') },
+    filename = function() { 
+		 paste(input$dataset, '.csv', sep='') 
+	 },
     content = function(file) {
       write.csv(datasetInput(), file)
     }
